/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Webserver Applikation

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "httpd.h"
#include "webpage.h"
#include "lcd.h"


struct http_table http_entry[MAX_TCP_ENTRY];

//Hier wird das codierte Passwort aus config.h gespeichert.
unsigned char http_auth_passwort[20];

unsigned char post_in[5] = {'O','U','T','='};
unsigned char post_ready[5] = {'S','U','B','='};
unsigned char PORT_tmp_C = 0;
unsigned char PORT_tmp_D = 0;

char time_string_Buffer1[10];
char time_string_Buffer2[10];
char time_string_Buffer3[10];
char time_string_Buffer4[10];
char time_string_Buffer5[10];
char time_string_Buffer6[10];
char time_string_Buffer7[10];
char time_string_Buffer8[10];

// unsigned char internal_message[10];

// -> main.c (Speicherplatz fr 1-wire Sensorwerte)
extern volatile int16_t ow_array[MAXSENSORS];

PROGMEM char http_header1[]={	"HTTP/1.0 200 Document follows\r\n"
								"Server: AVR_WEB_Switch\r\n"
								"Content-Type: text/html\r\n\r\n"};

PROGMEM char http_header2[]={	"HTTP/1.0 200 Document follows\r\n"
								"Server: AVR_WEB_Switch\r\n"
								"Content-Type: image/jpg\r\n\r\n"};

PROGMEM char http_header3[]={	"HTTP/1.0 401 Unauthorized\r\n"
								"Server: AVR_WEB_Switch\r\n"
								"WWW-Authenticate: Basic realm=\"NeedPassword\""
								"\r\nContent-Type: text/html\r\n\r\n"};

//----------------------------------------------------------------------------
//Kein Zugriff Seite bei keinem Passwort
PROGMEM char Page0[] = {"401 Unauthorized%END"};

unsigned char rx_header_end[5] = {"\r\n\r\n\0"};

//----------------------------------------------------------------------------
//Initialisierung des Httpd Testservers
void httpd_init (void)
{
	//HTTP_AUTH_STRING 
	decode_base64((unsigned char*)HTTP_AUTH_STRING,http_auth_passwort);

	//Serverport und Anwendung eintragen
	add_tcp_app (HTTPD_PORT, (void(*)(unsigned char))httpd);
}
   
//----------------------------------------------------------------------------
//http Testserver
void httpd (unsigned char index)
{
    //Verbindung wurde abgebaut!
    if (tcp_entry[index].status & FIN_FLAG)
    {
        return;
    }

	//Allererste Aufruf des Ports fr diese Anwendung
	//HTTPD_Anwendungsstack lschen
	if(tcp_entry[index].app_status==1)
	{
		httpd_stack_clear(index);
	}
	
	//HTTP wurde bei dieser Verbindung zum ersten mal aufgerufen oder
	//HTML Header Retransmission!
	if (tcp_entry[index].app_status <= 2)
	{	
		httpd_header_check (index);
		return;
	}
	
	//Der Header wurde gesendet und mit ACK besttigt (tcp_entry[index].app_status+1)
	//war das HTML Packet fertig, oder mssen weitere Daten gesendet werden, oder Retransmission?
	if (tcp_entry[index].app_status > 2 && tcp_entry[index].app_status < 0xFFFE)
	{
		httpd_data_send (index);
		return;
	}
	
	//Verbindung kann geschlossen werden! Alle HTML Daten wurden gesendet TCP Port kann
	//geschlossen werden (tcp_entry[index].app_status >= 0xFFFE)!!
	if (tcp_entry[index].app_status >= 0xFFFE)
	{
		tcp_entry[index].app_status = 0xFFFE;
		tcp_Port_close(index);
		return;
	}
	return;
}

//----------------------------------------------------------------------------
//HTTPD_STACK lschen
void httpd_stack_clear (unsigned char index)
{
	http_entry[index].http_header_type =0;
	http_entry[index].first_switch = 0;
	http_entry[index].http_auth = HTTP_AUTH_DEFAULT;
	http_entry[index].new_page_pointer = 0;
	http_entry[index].old_page_pointer = 0;
	http_entry[index].post = 0;
	http_entry[index].auth_ptr = http_auth_passwort;
	http_entry[index].post_ptr = post_in;
	http_entry[index].post_ready_ptr = post_ready;
	http_entry[index].hdr_end_pointer = rx_header_end;			
	HTTP_DEBUG("\r\n**** NEUE HTTP ANFORDERUNG ****\r\n\r\n");	
	return;
}

//----------------------------------------------------------------------------
//Eintreffenden Header vom Client checken
void httpd_header_check (unsigned char index)
{
	unsigned int a = 0;
	
	if(strcasestr_P((char*)&eth_buffer[TCP_DATA_START_VAR],PSTR("POST"))!=0)
		{
		http_entry[index].post = 1;
		}
	
	//finden der Authorization und das Ende im Header auch ber mehrere Packete hinweg!!	
	if(*http_entry[index].hdr_end_pointer != 0)
	{		
		for(a=TCP_DATA_START_VAR;a<(TCP_DATA_END_VAR);a++)
		{	
			HTTP_DEBUG("%c",eth_buffer[a]);
			
			if(!http_entry[index].http_auth) 
			{
				if (eth_buffer[a] != *http_entry[index].auth_ptr++)
				{
					http_entry[index].auth_ptr = http_auth_passwort;
				}
				if(*http_entry[index].auth_ptr == 0) 
				{
					http_entry[index].http_auth = 1;
					HTTP_DEBUG("  <---LOGIN OK!--->\r\n");
				}
			}
			
			if (eth_buffer[a] != *http_entry[index].hdr_end_pointer++)
			{
				http_entry[index].hdr_end_pointer = rx_header_end;
			}
			
			//Das Headerende wird mit (CR+LF+CR+LF) angezeigt!
			if(*http_entry[index].hdr_end_pointer == 0) 
			{
				HTTP_DEBUG("<---HEADER ENDE ERREICHT!--->\r\n");
				break;
			}
		}
	}
	
	//Einzelne Postpacket (z.B. bei firefox)
	if(http_entry[index].http_auth && http_entry[index].post == 1)
	{

		uint8_t schaltautomatik = 0;

		for(a = TCP_DATA_START_VAR;a<(TCP_DATA_END_VAR);a++)
		{	
			// Schaltanweisung finden!
			if (eth_buffer[a] != *http_entry[index].post_ptr++)
			{
				http_entry[index].post_ptr = post_in;
			}
			if(*http_entry[index].post_ptr == 0) 
			{
				switch (eth_buffer[a+1])
				  {
					  
				// Schaltanweisungen von PORTC1-7 auslesen 
				// und in PORT_tmp_C zusammenfgen
				// checkbox-value-Wert A-H in webpage.h
				case ('A'):
					PORT_tmp_C = PORT_tmp_C + 1;
					break;
				
				case ('B'):
					PORT_tmp_C = PORT_tmp_C + 2;
					break;
				
				case ('C'):
					PORT_tmp_C = PORT_tmp_C + 4;
					break;
					  
				case ('D'):
					PORT_tmp_C = PORT_tmp_C + 8;
					break;
					  
				case ('E'):
					PORT_tmp_C = PORT_tmp_C + 16;
					break;
					
				case ('F'):
					PORT_tmp_C = PORT_tmp_C + 32;
					break;
					  
				case ('G'):
					PORT_tmp_C = PORT_tmp_C + 64;
					break;
					  
				case ('H'):
					PORT_tmp_C = PORT_tmp_C + 128;
					break;


#if USE_AUTOMATIK				
				// Automatik ein/aus
				case ('X'):
					schaltautomatik = 1;
					break;
				// Schaltbedingungen (Temperatur, analoger Wert, Zeit, ...)
				// Fr jeden Port sind 2-4 Buchstaben reserviert, 
				// die je nach Bedarf in "webpage.h" als "value" bergeben werden knnen
				// C0 -> a
				// C1 -> b
				// C2 -> c
				// C3 -> d
				// C4 -> e
				// C5 -> f
				// C6 -> g
				// Zeit C0 -> I,J,K,L
				// Zeit C1 -> M,N,O,P
				// Zeit C2 -> Q,R,S,T
				// Zeit C3 -> U,V,W,Y
				// Zeit C4 -> Z,q,r,s
				// Zeit C5 -> t,u,v,w
				// Zeit C6 -> x,y,z,1
				// Zeit C7 -> 2,3,4,5
				// Nachttemperatur -> h
				// Nachttemperatur -> i
				// Nachttemperatur -> j
				// Nachttemperatur -> k
				// Nachttemperatur -> l
				// Nachttemperatur -> m
				// Nachttemperatur -> n
				// frei -> o,p,6,7,8,9,0

				// Einschalttemperaturen (min. 12 bis max. 30 Grad) setzen
				case ('a'):
					if ( var_array[10] < 30 ) {
						var_array[10]++;
					}
					if ( var_array[10] == 30 ) {
						var_array[10] = 12;
					}	
					break;
				case ('b'):
					if ( var_array[11] < 30 ) {
						var_array[11]++;
					}
					if ( var_array[11] == 30 ) {
						var_array[11] = 12;
					}	
					break;
				case ('c'):
					if ( var_array[12] < 30 ) {
						var_array[12]++;
					}
					if ( var_array[12] == 30 ) {
						var_array[12] = 12;
					}	
					break;	
				case ('d'):
					if ( var_array[13] < 30 ) {
						var_array[13]++;
					}
					if ( var_array[13] == 30 ) {
						var_array[13] = 12;
					}	
					break;
				case ('e'):
					if ( var_array[14] < 30 ) {
						var_array[14]++;
					}
					if ( var_array[14] == 30 ) {
						var_array[14] = 12;
					}	
					break;
				case ('f'):
					if ( var_array[15] < 30 ) {
						var_array[15]++;
					}
					if ( var_array[15] == 30 ) {
						var_array[15] = 12;
					}	
					break;
				case ('g'):
					if ( var_array[16] < 30 ) {
						var_array[16]++;
					}
					if ( var_array[16] == 30 ) {
						var_array[16] = 12;
					}	
					break;	

				// Schaltuhr 1 einstellen
				
				case ('I'):
					if ( var_array[18] < 23 )
						var_array[18] += 1;
					else 
						var_array[18] = 0;
					break;
				case ('J'):
					if ( var_array[19] < 45 )
						var_array[19] += 15;
					else 
						var_array[19] = 0;
					break;
				case ('K'):
					if ( var_array[20] < 23 )
						var_array[20] += 1;
					else 
						var_array[20] = 0;
					break;
				case ('L'):
					if ( var_array[21] < 45 )
						var_array[21] += 15;
					else 
						var_array[21] = 0;
					break;
				
				// Schaltuhr 2 einstellen
				case ('M'):
					if ( var_array[22] < 23 )
						var_array[22] += 1;
					else 
						var_array[22] = 0;
					break;
				case ('N'):
					if ( var_array[23] < 45 )
						var_array[23] += 15;
					else 
						var_array[23] = 0;
					break;
				case ('O'):
					if ( var_array[24] < 23 )
						var_array[24] += 1;
					else 
						var_array[24] = 0;
					break;
				case ('P'):
					if ( var_array[25] < 45 )
						var_array[25] += 15;
					else 
						var_array[25] = 0;
					break;

				// Schaltuhr 3 einstellen
				case ('Q'):
					if ( var_array[26] < 23 )
						var_array[26] += 1;
					else 
						var_array[26] = 0;
					break;
				case ('R'):
					if ( var_array[27] < 45 )
						var_array[27] += 15;
					else 
						var_array[27] = 0;
					break;
				case ('S'):
					if ( var_array[28] < 23 )
						var_array[28] += 1;
					else 
						var_array[28] = 0;
					break;
				case ('T'):
					if ( var_array[29] < 45 )
						var_array[29] += 15;
					else 
						var_array[29] = 0;
					break;
				
				// Schaltuhr 4 einstellen
				case ('U'):
					if ( var_array[30] < 23 )
						var_array[30] += 1;
					else 
						var_array[30] = 0;
					break;
				case ('V'):
					if ( var_array[31] < 45 )
						var_array[31] += 15;
					else 
						var_array[31] = 0;
					break;
				case ('W'):
					if ( var_array[32] < 23 )
						var_array[32] += 1;
					else 
						var_array[32] = 0;
					break;
				case ('Y'):
					if ( var_array[33] < 45 )
						var_array[33] += 15;
					else 
						var_array[33] = 0;
					break;
					
				// Schaltuhr 5 einstellen
				case ('Z'):
					if ( var_array[34] < 23 )
						var_array[34] += 1;
					else 
						var_array[34] = 0;
					break;
				case ('q'):
					if ( var_array[35] < 45 )
						var_array[35] += 15;
					else 
						var_array[35] = 0;
					break;
				case ('r'):
					if ( var_array[36] < 23 )
						var_array[36] += 1;
					else 
						var_array[36] = 0;
					break;
				case ('s'):
					if ( var_array[37] < 45 )
						var_array[37] += 15;
					else 
						var_array[37] = 0;
					break;
				
				// Schaltuhr 6 einstellen
				case ('t'):
					if ( var_array[38] < 23 )
						var_array[38] += 1;
					else 
						var_array[38] = 0;
					break;
				case ('u'):
					if ( var_array[39] < 45 )
						var_array[39] += 15;
					else 
						var_array[39] = 0;
					break;
				case ('v'):
					if ( var_array[40] < 23 )
						var_array[40] += 1;
					else 
						var_array[40] = 0;
					break;
				case ('w'):
					if ( var_array[41] < 45 )
						var_array[41] += 15;
					else 
						var_array[41] = 0;
					break;
					
				// Schaltuhr 7 einstellen
				case ('x'):
					if ( var_array[42] < 23 )
						var_array[42] += 1;
					else 
						var_array[42] = 0;
					break;
				case ('y'):
					if ( var_array[43] < 45 )
						var_array[43] += 15;
					else 
						var_array[43] = 0;
					break;
				case ('z'):
					if ( var_array[44] < 23 )
						var_array[44] += 1;
					else 
						var_array[44] = 0;
					break;
				case ('1'):
					if ( var_array[45] < 45 )
						var_array[45] += 15;
					else 
						var_array[45] = 0;
					break;
				
				// Schaltuhr 8 einstellen
				case ('2'):
					if ( var_array[46] < 23 )
						var_array[46] += 1;
					else 
						var_array[46] = 0;
					break;
				case ('3'):
					if ( var_array[47] < 45 )
						var_array[47] += 15;
					else 
						var_array[47] = 0;
					break;
				case ('4'):
					if ( var_array[48] < 23 )
						var_array[48] += 1;
					else 
						var_array[48] = 0;
					break;
				case ('5'):
					if ( var_array[49] < 45 )
						var_array[49] += 15;
					else 
						var_array[49] = 0;
					break;
			// Nachttemperaturen
				case ('h'):
					if ( var_array[50] < 30 ) {
						var_array[50]++;
					}
					if ( var_array[50] == 30 ) {
						var_array[50] = 8;
					}	
					break;
				case ('i'):
					if ( var_array[51] < 30 ) {
						var_array[51]++;
					}
					if ( var_array[51] == 30 ) {
						var_array[51] = 8;
					}	
					break;
				case ('j'):
					if ( var_array[52] < 30 ) {
						var_array[52]++;
					}
					if ( var_array[52] == 30 ) {
						var_array[52] = 8;
					}	
					break;	
				case ('k'):
					if ( var_array[53] < 30 ) {
						var_array[53]++;
					}
					if ( var_array[53] == 30 ) {
						var_array[53] = 8;
					}	
					break;
				case ('l'):
					if ( var_array[54] < 30 ) {
						var_array[54]++;
					}
					if ( var_array[54] == 30 ) {
						var_array[54] = 8;
					}	
					break;
				case ('m'):
					if ( var_array[55] < 30 ) {
						var_array[55]++;
					}
					if ( var_array[55] == 30 ) {
						var_array[55] = 8;
					}	
					break;
				case ('n'):
					if ( var_array[56] < 30 ) {
						var_array[56]++;
					}
					if ( var_array[56] == 30 ) {
						var_array[56] = 8;
					}	
					break;	
			


#endif // USE_AUTOMATIK

				  }
				http_entry[index].post_ptr = post_in;
				//Schaltanweisung wurde gefunden
			}
		
			//Submit schliet die suche ab!
			if (eth_buffer[a] != *http_entry[index].post_ready_ptr++)
			{
				http_entry[index].post_ready_ptr = post_ready;
			}
			if(*http_entry[index].post_ready_ptr == 0) 
			{
				http_entry[index].post = 0;

				// RoBue
				// Achtung PORTC -> PORTD
				// Schaltanweisung ausfhren = PORTD setzen
				PORTC = PORT_tmp_C;
		                PORT_tmp_C = 0;
#if USE_PORTD_SCHALT
				PORTD = PORT_tmp_D;
		                PORT_tmp_D = 0;
#endif // USE_PORTD_SCHALT

#if USE_AUTOMATIK
				var_array[9] = schaltautomatik;

#endif // USE_AUTOMATIK

				break;
				//Submit gefunden
			}
		}
	}

	
	
	//Welche datei wird angefordert? Wird diese in der Flashspeichertabelle gefunden?
	unsigned char page_index = 0;
	
	if (!http_entry[index].new_page_pointer)
	{
		for(a = TCP_DATA_START_VAR+5;a<(TCP_DATA_END_VAR);a++)
		{
			if (eth_buffer[a] == '\r')
			{
				eth_buffer[a] = '\0';
				break;
			}
		}
	
		while(WEBPAGE_TABLE[page_index].filename)
		{
			if (strcasestr((char*)&eth_buffer[TCP_DATA_START_VAR],WEBPAGE_TABLE[page_index].filename)!=0) 
				{
					http_entry[index].http_header_type = 1;
					HTTP_DEBUG("\r\n\r\nDatei gefunden: ");
					HTTP_DEBUG("%s",(char*)WEBPAGE_TABLE[page_index].filename);
					HTTP_DEBUG("<----------------\r\n\r\n");	
					if (strcasestr(WEBPAGE_TABLE[page_index].filename,".jpg")!=0)
					{
						http_entry[index].http_header_type = 1;
					}
					if (strcasestr(WEBPAGE_TABLE[page_index].filename,".gif")!=0)
					{
						http_entry[index].http_header_type = 1;
					}	
					if (strcasestr(WEBPAGE_TABLE[page_index].filename,".htm")!=0)
					{
						http_entry[index].http_header_type = 0;	
					}	
					http_entry[index].new_page_pointer = WEBPAGE_TABLE[page_index].page_pointer;
					break;
				}
			page_index++;
		}
	}

	//Wurde das Ende vom Header nicht erreicht
	//kommen noch weitere Stcke vom Header!
	if ((*http_entry[index].hdr_end_pointer != 0) || (http_entry[index].post == 1))
	{
		//Der Empfang wird Quitiert und es wird auf weiteres Headerstck gewartet
		tcp_entry[index].status =  ACK_FLAG;
		create_new_tcp_packet(0,index);
		//Warten auf weitere Headerpackete
		tcp_entry[index].app_status = 1;
		return;
	}	
	
	//Wurde das Passwort in den ganzen Headerpacketen gefunden?
	//Wenn nicht dann ausfhren und Passwort anfordern!
	if((!http_entry[index].http_auth) && tcp_entry[index].status&PSH_FLAG)
	{	
		//HTTP_AUTH_Header senden!
		http_entry[index].new_page_pointer = Page0;
		memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header3,(sizeof(http_header3)-1));
		tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
		create_new_tcp_packet((sizeof(http_header3)-1),index);
		tcp_entry[index].app_status = 2;
		return;
	}
	
	//Standart INDEX.HTM Seite wenn keine andere gefunden wurde
	if (!http_entry[index].new_page_pointer)
	{
		//Besucher Counter
		var_array[MAX_VAR_ARRAY-1]++;
			// Counter zuruecksetzen
			if (var_array[MAX_VAR_ARRAY-1] == 10000)
				var_array[MAX_VAR_ARRAY-1] = 0;
		http_entry[index].new_page_pointer = Page1;
		http_entry[index].http_header_type = 0;
	}	
	
	tcp_entry[index].app_status = 2;
	//Seiten Header wird gesendet
	if(http_entry[index].http_header_type == 1)
	{
		memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header2,(sizeof(http_header2)-1));
        tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
        create_new_tcp_packet((sizeof(http_header2)-1),index);
        return;
	}
     
	if(http_entry[index].http_header_type == 0)
	{
		memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header1,(sizeof(http_header1)-1));
        tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
        create_new_tcp_packet((sizeof(http_header1)-1),index);
        return;
	}
    return;
}

//----------------------------------------------------------------------------
//Daten Packete an Client schicken
void httpd_data_send (unsigned char index)
{	
	unsigned int a;
	unsigned char str_len;
	
	char var_conversion_buffer[CONVERSION_BUFFER_LEN];
	
	//Passwort wurde im Header nicht gefunden
	if(!http_entry[index].http_auth)
	{
		http_entry[index].new_page_pointer = Page0;
	}
	

	//kein Packet empfangen Retransmission des alten Packetes
	if (tcp_entry[index].status == 0) 
	{
		http_entry[index].new_page_pointer = http_entry[index].old_page_pointer;
	}
	http_entry[index].old_page_pointer = http_entry[index].new_page_pointer;

	for (a = 0;a<(MTU_SIZE-(TCP_DATA_START)-150);a++)
	{
		unsigned char b;
		b = pgm_read_byte(http_entry[index].new_page_pointer++);
		eth_buffer[TCP_DATA_START + a] = b;
		
		//Mssen Variablen ins Packet eingesetzt werden? ===> %VA@00 bis %VA@19
		if (b == '%')
		{
			if (strncasecmp_P("VA@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10;
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	
				itoa (var_array[b],var_conversion_buffer,10);
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}


			// Zeitstring 1
			if (strncasecmp_P("TI@",http_entry[index].new_page_pointer,3)==0)
			{
				//unsigned char hh = (time/3600)%24;
				itoa (hh,var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer1,var_conversion_buffer);
        			strcat (time_string_Buffer1,":");
				if ( mm < 10 ) {
					strcat (time_string_Buffer1,"0");
						}
        			itoa (mm,var_conversion_buffer,10);
        			strcat(time_string_Buffer1,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer1,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer1,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;
			}

			// Schaltzeit 1
			if (strncasecmp_P("T0@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[18],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer1,var_conversion_buffer);
        			strcat (time_string_Buffer1,":");
				if ( var_array[19] < 10 ) {
					strcat (time_string_Buffer1,"0");
						}
        			itoa (var_array[19],var_conversion_buffer,10);
        			strcat(time_string_Buffer1,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer1,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer1,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;	
			}
			if (strncasecmp_P("T1@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[20],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer1,var_conversion_buffer);
        			strcat (time_string_Buffer1,":");
				if ( var_array[21] < 10 ) {
					strcat (time_string_Buffer1,"0");
						}
        			itoa (var_array[21],var_conversion_buffer,10);
        			strcat(time_string_Buffer1,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer1,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer1,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;		
			}				


			// Zeitstring 2
			if (strncasecmp_P("TI@",http_entry[index].new_page_pointer,3)==0)
			{
				//unsigned char hh = (time/3600)%24;
				itoa (hh,var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer2,var_conversion_buffer);
        			strcat (time_string_Buffer2,":");
				if ( mm < 10 ) {
					strcat (time_string_Buffer2,"0");
						}
        			itoa (mm,var_conversion_buffer,10);
        			strcat(time_string_Buffer2,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer2,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer2,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;
			}

			// Schaltzeit 2
			if (strncasecmp_P("T2@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[22],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer2,var_conversion_buffer);
        			strcat (time_string_Buffer2,":");
				if ( var_array[23] < 10 ) {
					strcat (time_string_Buffer2,"0");
						}
        			itoa (var_array[23],var_conversion_buffer,10);
        			strcat(time_string_Buffer2,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer2,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer2,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;	
			}
			if (strncasecmp_P("T3@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[24],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer2,var_conversion_buffer);
        			strcat (time_string_Buffer2,":");
				if ( var_array[25] < 10 ) {
					strcat (time_string_Buffer2,"0");
						}
        			itoa (var_array[25],var_conversion_buffer,10);
        			strcat(time_string_Buffer2,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer2,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer2,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;		
			}				

			// Zeitstring 3
			if (strncasecmp_P("TI@",http_entry[index].new_page_pointer,3)==0)
			{
				//unsigned char hh = (time/3600)%24;
				itoa (hh,var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer3,var_conversion_buffer);
        			strcat (time_string_Buffer3,":");
				if ( mm < 10 ) {
					strcat (time_string_Buffer3,"0");
						}
        			itoa (mm,var_conversion_buffer,10);
        			strcat(time_string_Buffer3,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer3,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer3,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;
			}

			// Schaltzeit 3
			if (strncasecmp_P("T4@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[26],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer3,var_conversion_buffer);
        			strcat (time_string_Buffer3,":");
				if ( var_array[27] < 10 ) {
					strcat (time_string_Buffer3,"0");
						}
        			itoa (var_array[27],var_conversion_buffer,10);
        			strcat(time_string_Buffer3,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer3,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer3,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;	
			}
			if (strncasecmp_P("T5@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[28],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer3,var_conversion_buffer);
        			strcat (time_string_Buffer3,":");
				if ( var_array[29] < 10 ) {
					strcat (time_string_Buffer3,"0");
						}
        			itoa (var_array[29],var_conversion_buffer,10);
        			strcat(time_string_Buffer3,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer3,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer3,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;		
			}				





			// Zeitstring 4
			if (strncasecmp_P("TI@",http_entry[index].new_page_pointer,3)==0)
			{
				//unsigned char hh = (time/3600)%24;
				itoa (hh,var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer4,var_conversion_buffer);
        			strcat (time_string_Buffer4,":");
				if ( mm < 10 ) {
					strcat (time_string_Buffer4,"0");
						}
        			itoa (mm,var_conversion_buffer,10);
        			strcat(time_string_Buffer4,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer4,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer4,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;
			}

			// Schaltzeit 4
			if (strncasecmp_P("T6@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[30],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer4,var_conversion_buffer);
        			strcat (time_string_Buffer4,":");
				if ( var_array[31] < 10 ) {
					strcat (time_string_Buffer4,"0");
						}
        			itoa (var_array[31],var_conversion_buffer,10);
        			strcat(time_string_Buffer4,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer4,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer4,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;	
			}
			if (strncasecmp_P("T7@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[32],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer4,var_conversion_buffer);
        			strcat (time_string_Buffer4,":");
				if ( var_array[33] < 10 ) {
					strcat (time_string_Buffer4,"0");
						}
        			itoa (var_array[33],var_conversion_buffer,10);
        			strcat(time_string_Buffer4,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer4,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer4,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;		
			}	

			// Zeitstring 5
			if (strncasecmp_P("TI@",http_entry[index].new_page_pointer,3)==0)
			{
				//unsigned char hh = (time/3600)%24;
				itoa (hh,var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer5,var_conversion_buffer);
        			strcat (time_string_Buffer5,":");
				if ( mm < 10 ) {
					strcat (time_string_Buffer5,"0");
						}
        			itoa (mm,var_conversion_buffer,10);
        			strcat(time_string_Buffer5,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer5,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer5,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;
			}

			// Schaltzeit 5
			if (strncasecmp_P("T8@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[34],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer5,var_conversion_buffer);
        			strcat (time_string_Buffer5,":");
				if ( var_array[35] < 10 ) {
					strcat (time_string_Buffer5,"0");
						}
        			itoa (var_array[35],var_conversion_buffer,10);
        			strcat(time_string_Buffer5,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer5,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer5,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;	
			}
			if (strncasecmp_P("T9@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[36],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer5,var_conversion_buffer);
        			strcat (time_string_Buffer5,":");
				if ( var_array[37] < 10 ) {
					strcat (time_string_Buffer5,"0");
						}
        			itoa (var_array[37],var_conversion_buffer,10);
        			strcat(time_string_Buffer5,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer5,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer5,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;		
			}	


			// Zeitstring 6
			if (strncasecmp_P("TI@",http_entry[index].new_page_pointer,3)==0)
			{
				//unsigned char hh = (time/3600)%24;
				itoa (hh,var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer6,var_conversion_buffer);
        			strcat (time_string_Buffer6,":");
				if ( mm < 10 ) {
					strcat (time_string_Buffer6,"0");
						}
        			itoa (mm,var_conversion_buffer,10);
        			strcat(time_string_Buffer6,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer6,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer6,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;
			}

			// Schaltzeit 6
			if (strncasecmp_P("T10@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[38],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer6,var_conversion_buffer);
        			strcat (time_string_Buffer6,":");
				if ( var_array[39] < 10 ) {
					strcat (time_string_Buffer6,"0");
						}
        			itoa (var_array[39],var_conversion_buffer,10);
        			strcat(time_string_Buffer6,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer6,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer6,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;	
			}
			if (strncasecmp_P("T11@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[40],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer6,var_conversion_buffer);
        			strcat (time_string_Buffer6,":");
				if ( var_array[41] < 10 ) {
					strcat (time_string_Buffer6,"0");
						}
        			itoa (var_array[41],var_conversion_buffer,10);
        			strcat(time_string_Buffer6,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer6,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer6,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;		
			}	

			// Zeitstring 7
			if (strncasecmp_P("TI@",http_entry[index].new_page_pointer,3)==0)
			{
				//unsigned char hh = (time/3600)%24;
				itoa (hh,var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer7,var_conversion_buffer);
        			strcat (time_string_Buffer7,":");
				if ( mm < 10 ) {
					strcat (time_string_Buffer7,"0");
						}
        			itoa (mm,var_conversion_buffer,10);
        			strcat(time_string_Buffer7,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer7,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer7,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;
			}

			// Schaltzeit 7
			if (strncasecmp_P("T12@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[42],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer7,var_conversion_buffer);
        			strcat (time_string_Buffer7,":");
				if ( var_array[43] < 10 ) {
					strcat (time_string_Buffer7,"0");
						}
        			itoa (var_array[43],var_conversion_buffer,10);
        			strcat(time_string_Buffer7,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer7,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer7,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;	
			}
			if (strncasecmp_P("T13@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[44],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer7,var_conversion_buffer);
        			strcat (time_string_Buffer7,":");
				if ( var_array[45] < 10 ) {
					strcat (time_string_Buffer7,"0");
						}
        			itoa (var_array[45],var_conversion_buffer,10);
        			strcat(time_string_Buffer7,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer7,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer7,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;		
			}	

			// Zeitstring 8
			if (strncasecmp_P("TI@",http_entry[index].new_page_pointer,3)==0)
			{
				//unsigned char hh = (time/3600)%24;
				itoa (hh,var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer8,var_conversion_buffer);
        			strcat (time_string_Buffer8,":");
				if ( mm < 10 ) {
					strcat (time_string_Buffer8,"0");
						}
        			itoa (mm,var_conversion_buffer,10);
        			strcat(time_string_Buffer8,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer8,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer8,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;
			}

			// Schaltzeit 8
			if (strncasecmp_P("T14@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[46],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer8,var_conversion_buffer);
        			strcat (time_string_Buffer8,":");
				if ( var_array[47] < 10 ) {
					strcat (time_string_Buffer8,"0");
						}
        			itoa (var_array[47],var_conversion_buffer,10);
        			strcat(time_string_Buffer8,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer8,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer8,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;	
			}
			if (strncasecmp_P("T15@",http_entry[index].new_page_pointer,3)==0)
			{	
			itoa (var_array[48],var_conversion_buffer,10);
 	      			strcpy(time_string_Buffer8,var_conversion_buffer);
        			strcat (time_string_Buffer8,":");
				if ( var_array[49] < 10 ) {
					strcat (time_string_Buffer8,"0");
						}
        			itoa (var_array[49],var_conversion_buffer,10);
        			strcat(time_string_Buffer8,var_conversion_buffer);
               			str_len = strnlen(time_string_Buffer8,10);
        			memmove(&eth_buffer[TCP_DATA_START+a],time_string_Buffer8,str_len);
	       			a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+3;		
			}	




#if USE_OW
	
			/*
			*	1-Wire Temperatursensoren
			*	-------------------------
			*	OW@nn	nn = 00 bis MAXSENSORS-1 gibt Werte in 1/10 C aus
			*	OW@mm	mm = 20 bis MAXSENSORS-1+20 gibt Werte in C mit einer Nachkommastelle aus
			*	d.h. OW@nn fr Balkenbreite verwenden und OW@mm fr Celsius-Anzeige
			*/
			if (strncasecmp_P("OW@",http_entry[index].new_page_pointer,3)==0)	
			{
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10;
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);
					
					// Wert auslesen
					int16_t ow_temp = ow_array[b];
					str_len = 0;
	
					// evtl. Vorzeichen einfuegen:	
					if ( ow_temp < 0 ) {
						ow_temp *= (-1);
						var_conversion_buffer[0] = '-';
						memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,1);
						a ++;
					}
					
					// Wert vor dem Komma einfgen
					itoa (ow_temp/10,var_conversion_buffer,10);
					str_len += strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
					
					// Komma einfgen
					var_conversion_buffer[str_len] = ',';
					str_len++;
					memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
					a += str_len;

					// Wert nach dem Komma einfgen					
					itoa (ow_temp%10,var_conversion_buffer,10);
					str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
					memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
					a += str_len-1;

					http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;

			}

#endif	// USE_OW

			//Einsetzen des Port Status %PORTxy durch "checked" wenn Portx.Piny = 1
			//x: A..G  y: 0..7 
			if (strncasecmp_P("PORT",http_entry[index].new_page_pointer,4)==0)
			{
				unsigned char pin  = (pgm_read_byte(http_entry[index].new_page_pointer+5)-48);	
				b = 0;
				switch(pgm_read_byte(http_entry[index].new_page_pointer+4))
				{
					case 'A':
						b = (PORTA & (1<<pin));
						break;
					case 'B':
						b = (PORTB & (1<<pin));
						break;
					case 'C':
						b = (PORTC & (1<<pin));
						break;
					case 'D':
						b = (PORTD & (1<<pin));
						break;
					// Checkbox Automatik
					case 'X':
						b = var_array[9];
						break; 
				}
				
				if(b)
				{
					strcpy_P(var_conversion_buffer, PSTR("checked"));
				}
				else
				{
					strcpy_P(var_conversion_buffer, PSTR("\0"));
				}
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a += str_len-1;
				http_entry[index].new_page_pointer = http_entry[index].new_page_pointer+6;
			}
						
			// Einsetzen des Pin Status %PI@xy bis %PI@xy 
			// durch grne oder graue Hintergrundarbe der Tabellenzelle

			//x = 0 : PINA / x = 1 : PINB / x = 2 : PINC / x = 3 : PIND

			if (strncasecmp_P("PIN",http_entry[index].new_page_pointer,3)==0)
			{
				unsigned char pin  = (pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	
				b = 0;
				switch(pgm_read_byte(http_entry[index].new_page_pointer+3))
				{
					case 'A':
						b = (PINA & (1<<pin));
						break;
					case 'B':
						b = (PINB & (1<<pin));
						break;
					case 'C':
						b = (PINC & (1<<pin));
						break;
					case 'D':
						b = (PIND & (1<<pin));
						break;    
				}
				
				if(b)
				{
					strcpy_P(var_conversion_buffer, PSTR("ledon.gif"));
				}
				else
				{
					strcpy_P(var_conversion_buffer, PSTR("ledoff.gif"));
				}
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a += str_len-1;
				http_entry[index].new_page_pointer = http_entry[index].new_page_pointer+5;
			}
			//wurde das Ende des Packetes erreicht?
			//Verbindung TCP Port kann beim nchsten ACK geschlossen werden
			//Schleife wird abgebrochen keine Daten mehr!!
			if (strncasecmp_P("END",http_entry[index].new_page_pointer,3)==0)
			{	
				tcp_entry[index].app_status = 0xFFFD;
				break;
			}
		}
	}
	//Erzeugte Packet kann nun gesendet werden!
	tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
	create_new_tcp_packet(a,index);
	return;
}
